# -*- coding: utf-8 -*-
##############################################
# The MIT License (MIT)
# Copyright (c) 2014 Kevin Walchko
# see LICENSE for full details
##############################################
from subprocess import check_output  # call command line
import os              # make directories, change current dir, etc
import platform        # macOS or windows
import shutil          # move and delete files/folders
import pathlib         # recursive file finding
from math import ceil  # file size
# from colorama import Fore


def run(cmd):
    """Runs a command string and returns the output"""
    cmds = cmd.split()
    return check_output(cmds).strip().decode("utf-8")


def mkdir(path):
    """Makes a directory"""
    try:
        os.mkdir(path)
    except FileExistsError:
        # folder was already created ... it's ok
        pass


def rmdir(path):
    """Removes (deletes) a directory"""
    try:
        shutil.rmtree(path)
    except FileNotFoundError:
        # folder was already deleted or doesn't exist ... it's ok
        pass

def rm(fname):
    """Removes (deletes) a file or list of files"""
    if fname is None:
        # print(f"{Fore.RED}*** No file to remove ***{Fore.RESET}")
        raise Exception("No file name given")
        return
    if not isinstance(fname, list):
        fname = [fname]
    for f in fname:
        try:
            os.remove(f)
            # print(f"{Fore.RED}- {f}{Fore.RESET}")
        except FileNotFoundError:
            # folder was already deleted or doesn't exist ... it's ok
            pass


def find(path, fname):
    """
    Given a path, this will recursively search for a file (bob.txt) or
    pattern (*.txt). It returns an array of found file paths.
    """
    fn = []
    for p in pathlib.Path(path).rglob(fname):
        fn.append(p)
    return fn


def touch(file):
    """Given a file name, operates like the Unix touch command"""
    pathlib.Path(file).touch()


def file_size(s):
    """Given a file size in bytes, this returns a printable size

    Returns: (size, prefix) - > (12.345, 'GB')
    """
    s = ceil(s/8)
    mem = ['B', 'KB', 'MB', 'GB', 'TB', 'PB']
    for x in range(1, 6):
        if s > 1000:
            s = ceil(s / 1024)
            size = mem[x]
    return s, size



# -*- coding: utf-8 -*-
##############################################
# The MIT License (MIT)
# Copyright (c) 2014 Kevin Walchko
# see LICENSE for full details
##############################################

from .sig import SignalCatch
from .rate import Rate
from .simple_process import SimpleProcess

from importlib.metadata import version # type: ignore

__author__ = 'Kevin Walchko'
__license__ = 'MIT'
__version__ = version("slurm")



# -*- coding: utf-8 -*-
##############################################
# The MIT License (MIT)
# Copyright (c) 2014 Kevin Walchko
# see LICENSE for full details
##############################################
import pickle as dill
from pathlib import Path
import gzip


def compression(path):
    """
    Based on filename, is compression being used?
    """
    compress = None
    if not isinstance(path, Path):
        path = Path(path)

    ext = path.suffix

    if ext == ".gz":
        compress = True
    elif ext == ".pkl":
        compress = False
    else:
        raise Exception(f"invalid file extension [{ext}], must be .pkl or .pkl.gz")

    return compress

def write(info, data, fname, compressionlevel=3):
    """
    Stores data in a binary file using dill. This is better than pickle when
    using namedtuples to store data.

    info: dict holding sensor info/settings
    data: sensor data, [{},{},...,{}]
    fname: file name, file extension: *.dil=uncompressess, *.dil.gz=compressed
    compressionlevel: [low,fastest] 1-9 [highest,slowest]
    """
    bag = {
        "info": info,
        "data": data
    }

    path = Path(fname).expanduser()
    compress = compression(path)

    with path.open("wb") as fd:
        if compress:
            d = dill.dumps(bag)
            d = gzip.compress(d, compresslevel=compressionlevel)
            fd.write(d)
        else:
            dill.dump(bag, fd)

def read(fname):
    """
    Opens and reads a data file.

    fname: file name, file extension: *.dil=uncompressess, *.dil.gz=compressed
    """
    path = Path(fname).expanduser()
    compress = compression(path)

    with path.open("rb") as fd:
        if compress:
            d = fd.read()
            d = gzip.decompress(d)
            data = dill.loads(d)
        else:
            data = dill.load(fd)

    return data



# -*- coding: utf-8 -*-
##############################################
# The MIT License (MIT)
# Copyright (c) 2018 Kevin Walchko
# see LICENSE for full details
##############################################
import multiprocessing as mp
# from colorama import Fore
# import attr


# @attr.s(slots=True)
class SimpleProcess:
    """
    A simple class to help processes start/stop easily. It is main intended for
    testing and some simple things.

    .. code-block:: python

        p = SimpleProcess()       # create process
        p.start(function)         # start the process, runs function
        ...                       # stuff happens
        p.join()                  # time to go ... bye!
    """
    # _ps = attr.ib(init=False, default=None)
    _ps = None

    def __del__(self):
        if self._ps:
            self.join(0.1)

    @property
    def name(self):
        return self._ps.name

    @property
    def pid(self):
        return self._ps.pid

    def is_alive(self):
        """Check if the process is still running"""
        if self._ps:
            return self._ps.is_alive()
        else:
            return False

    def terminate(self):
        """Terminate the process"""
        if self._ps:
            self._ps.terminate()

    def start(self, func, name='simple_process', **kwargs):
        """Starts the process
          :func: function for multi-process
          :name: what to call the process
          :kwargs: args to pass function
        """
        if kwargs:
            kwargs = kwargs['kwargs']  # WTF???
            self._ps = mp.Process(name=name, target=func, kwargs=kwargs)
        else:
            self._ps = mp.Process(name=name, target=func)

        self._ps.start()
        # print(f'>> {Fore.GREEN}Started{Fore.RESET}: {self._ps.name}[{self._ps.pid}]')

    def join(self, timeout=1.0):
        """
        Attempts to join() the process with the given timeout. If that fails, it calls
        terminate().

            :timeout: how long to wait for join() in seconds.
        """
        # print(f'>> {Fore.RED}Stopping{Fore.RESET}: {self._ps.name}[{self._ps.pid}] ...')
        if self._ps:
            self._ps.join(timeout)
            if self._ps.is_alive():
                self._ps.terminate()
        self._ps = None



# -*- coding: utf-8 -*-
##############################################
# The MIT License (MIT)
# Copyright (c) 2014 Kevin Walchko
# see LICENSE for full details
##############################################
import time


class Rate:
    """
    Uses sleep to keep a desired message/sample rate for a loop.
    """
    def __init__(self, hertz):
        """
            :hertz: rate loop should run at
        """
        self.last_time = time.time()
        self.dt = 1/hertz

    def sleep(self):
        """
        This uses sleep to delay the function. If your loop is faster than your
        desired Hertz, then this will calculate the time difference so sleep
        keeps you close to you desired hertz. If your loop takes longer than
        your desired hertz, then it doesn't sleep.
        """
        now = time.time()
        diff = now - self.last_time
        if diff < self.dt:
            new_sleep = self.dt - diff
            time.sleep(new_sleep)

        # now that we hav slept a while, set the current time
        # as the last time
        self.last_time = time.time()



# -*- coding: utf-8 -*-
##############################################
# The MIT License (MIT)
# Copyright (c) 2020 Kevin Walchko
# see LICENSE for full details
##############################################
from types import ModuleType as MT

def importedModules():
    """
    Determines the version of imported modules if it can. Unfortunately
    some authors do strange things that makes this simple process more
    difficult.
    """
    s = set()
    libs = {}
    for k,v in globals().items():
        if type(v) is MT and not k.startswith('__'):
            n = v.__package__
            if n is None:
                n = v.__name__
            n = n.split('.')[0]
            
            if n in ["spatialmath","roboticstoolbox"]:
                n += "-python"

            if n != '':
                s.add(n)

    for n in s:
        try:
            libs[n] = version(n)
        except:
            libs[n] = "unknown"
            
    return libs



# -*- coding: utf-8 -*-
##############################################
# The MIT License (MIT)
# Copyright (c) 2014 Kevin Walchko
# see LICENSE for full details
##############################################
import signal
# from colorama import Back


class SignalCatch:
    """
    Catches SIGINT and SIGTERM signals and sets kill = True

    .. code-block:: bash

        $ kill -l
         1) SIGHUP     2) SIGINT       3) SIGQUIT     4) SIGILL
         5) SIGTRAP    6) SIGABRT      7) SIGEMT      8) SIGFPE
         9) SIGKILL    10) SIGBUS     11) SIGSEGV    12) SIGSYS
        13) SIGPIPE    14) SIGALRM    15) SIGTERM    16) SIGURG
        17) SIGSTOP    18) SIGTSTP    19) SIGCONT    20) SIGCHLD
        21) SIGTTIN    22) SIGTTOU    23) SIGIO      24) SIGXCPU
        25) SIGXFSZ    26) SIGVTALRM  27) SIGPROF    28) SIGWINCH
        29) SIGINFO    30) SIGUSR1    31) SIGUSR2

    `stackoverflow <https://stackoverflow.com/questions/18499497/how-to-process-sigterm-signal-gracefully>`_
    """
    _kill = False
    kill_init = False

    @property
    def kill(self):
        """Returns the kill value"""
        return self._kill

    @kill.setter
    def kill(self, val):
        """Sets the kill value"""
        self._kill = val
        # print("SIGNAL CHANGED KILL:", self._kill)
        # raise Exception("WTF")

    def kill_signals(self):
        if self.kill_init:
            # print("*** KILL ALREADY SET ***")
            return
        signal.signal(signal.SIGINT, self._exit_gracefully)
        signal.signal(signal.SIGTERM, self._exit_gracefully)
        self.kill_init = True

    def exit_gracefully(self, signum):
        """
        When handler gets called, it sets the self.kill to True
        """
        if signum == 2:
            self.kill = True
            # print(Back.RED + ">> Got signal[{}]".format(signum) + Style.RESET_ALL)
            # print(f"{Back.RED}>> Got signal[{signum}]{Back.RESET_ALL}")
        # else:
        #     # print(Back.YELLOW + ">> Got signal[{}]".format(signum) + Style.RESET_ALL)
        #     print(f"{Back.YELLOW}>> Got signal[{signum}]{Back.RESET_ALL}")



# -*- coding: utf-8 -*-
##############################################
# The MIT License (MIT)
# Copyright (c) 2014 Kevin Walchko
# see LICENSE for full details
##############################################
import socket
import platform


def get_ip():
    """
    Gets the localhost's ip address and returns:
        - ip address
        - 127.0.0.1
    """
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    IP = None
    try:
        # doesn't even have to be reachable
        s.connect(('10.255.255.255', 1))
        IP = s.getsockname()[0]
    except Exception:
        try:
            n = socket.gethostname()
            # make sure it has a zeroconfig .local or you end up
            # with 127.0.0.1 as your address
            if n.find('.local') < 0:
                n += '.local'
            IP = socket.gethostbyname(n)
        except Exception:
            IP = '127.0.0.1'
    finally:
        s.close()

    return IP

def get_host():
    """Returns (hostname, ipaddr)"""
    return (platform.node(), get_ip(),)



# -*- coding: utf-8 -*-
##############################################
# The MIT License (MIT)
# Copyright (c) 2014 Kevin Walchko
# see LICENSE for full details
##############################################

import yaml
import pickle
import os
try:
    import simplejson as json
except ImportError:
    import json


def get_size(fname):
    """Returns the file size"""
    return os.path.getsize(fname)

def _storage_read(fname, func, access='r'):
    fname = os.path.expanduser(fname)
    # print(fname)
    try:
        with open(fname, access) as fd:
            data = func(fd)

    except IOError:
        raise Exception(f'Could not open {fname} for reading')

    return data

def _storage_write(fname, func, data, access='w', comments=None, compress=False):
    fname = os.path.expanduser(fname)
    # print(fname)
    try:
        with open(fname, access) as fd:
            if comments:
                fd.write(comments + "\n")
            func(data, fd)

    except IOError:
        raise Exception(f'Could not open {fname} for writing')


def write(fname, data, fmt=None, comments=None):
    """
    Writes a Pickle, Yaml or Json file
        :filename: file name
        :data: data to be written
        :fmt: [optional] format (pickle, yaml or json)
    """
    if not fmt:
        fmt = fname.split(".")[-1]

    if fmt in ['yml', 'yaml']:
        _storage_write(fname, yaml.safe_dump, data, comments=comments)
    elif fmt == "json":
        _storage_write(fname, json.dump, data)
    elif fmt == "pickle":
        _storage_write(fname, pickle.dump, data, 'wb')
    elif fmt == "pickle.gz":
        _storage_write(fname, pickle.dump, data, 'wb', compress=True)
    else:
        raise Exception(f"Unknown format: {fmt}")

def read(fname, fmt=None):
    """
    Reads a Pickle, Yaml or Json file
        :fname: file name
        :fmt: [optional] format (pickle, yaml or json)
    return
        :data: dictionary of data
    """
    if not fmt:
        fmt = fname.split(".")[-1]

    if fmt in ['yml', 'yaml']:
        return _storage_read(fname, yaml.safe_load)
    elif fmt == "json":
        return _storage_read(fname, json.load)
    elif fmt == "pickle":
        return _storage_read(fname, pickle.load, 'rb')
    else:
        raise Exception()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['slurm']

package_data = \
{'': ['*']}

install_requires = \
['pyyaml']

setup_kwargs = {
    'name': 'slurm',
    'version': '0.5.1',
    'description': 'A bunch tools I have created over the years',
    'long_description': '![](https://github.com/MomsFriendlyRobotCompany/slurm/blob/master/pics/slurm.jpg?raw=true)\n\n# Slurm\n\n\n[![Actions Status](https://github.com/MomsFriendlyRobotCompany/slurm/workflows/walchko%20pytest/badge.svg)](https://github.com/MomsFriendlyRobotCompany/slurm/actions)\n![GitHub](https://img.shields.io/github/license/MomsFriendlyRobotCompany/slurm)\n![PyPI - Python Version](https://img.shields.io/pypi/pyversions/slurm)\n![PyPI](https://img.shields.io/pypi/v/slurm)\n![PyPI - Downloads](https://img.shields.io/pypi/dm/slurm?color=aqua)\n\nThis is a collection of tools I have used over the years collected together.\n\n## Signal Catcher\n\n`SignalCatch` catches `SIGINT` and `SIGTERM` signals and sets\n`SignalCatch.kill` to `True`.\n\n```python\nfrom slurm import SignalCatch\n\nsig = SignalCatch()\n\nwhile True:\n    if sig.kill == True:\n        exit(0)\n```\n\n## Simple Processes\n\n```python\nfrom slurm import SimpleProcess\n\ndef func():\n    # some simple process that does something\n    for _ in range(10):\n        print(".", end="")\n        time.sleep(0.1)\n    print("")\n\ndef test_process():\n    p = SimpleProcess()\n    p.start(func)\n    print(p)\n    p.join(timeout=2.0) # if not ended in 2 sec, will terminate() the process\n```\n\n## Storage\n\n```python\nfrom slurm import storage\n\npick = storage.read("file.pickle")\nyaml = storage.read("file.yaml")\njson = storage.read("file.json")\njson = storage.read("file", "json")\n\n\ndata = [1,2,3,4]\nstorage.write("tom.pickle", data)\nstorage.write("bob.json", data)\nstorage.write("guess.file", data, "yml")\n```\n\nAlso, for YAML files, you can put comments in:\n\n```python\ninfo = {\n    "a": 1\n}\n\nnum = 5\ncomm = f"""\n# hello {num} dogs!!\n# there\n# big boy\n"""\nstorage.write("t.yaml", info, comments=comm)\n```\n\nwhich will produce:\n\n```yaml\n# hello 5 dogs!!\n# there\n# big boy\n\na: 1\n```\n\n## Science Storage\n\nOver the years I have collected a lot of data, but not completely documented\nthe sensors or their settings. I am trying to setup a data file that can:\n\n- use primarly standard python libraries to read data files\n- self documenting with info and `namedtuples`\n- can use `gzip` for compression of large files\n\n```python\nfrom slurm import scistorage\nfrom collections import namedtuple\n\nSensor = namedtuple("Sensor","x y z")\n\n# document sensor setting in this data file\n# there is no real format for this, just put good\n# stuff here\ninfo = {\n    "TFmini": {\n        "min": 0.3,\n        "max": 12.0,\n        "fov_deg": 4.6,\n        "units": "m"\n    },\n    "LSM6DSOX": {\n        "accel": {\n            "range": (-4,4),\n            "units": "g"\n        },\n        "gyro": {\n            "range": (-2000,2000),\n            "units": "dps"\n        }\n    },\n    "LIS3MDL": {\n        "range": (-4,4), # 4 gauss = 400 uT\n        "units": "gauss"\n    },\n    "DPS310": {\n        "sensors": ("temperature", "pressure")\n    }\n}\n\ndata = [] # some data stored in an array or deque\nfor i in range(100):\n    data.append(Sensor(i,i,i)) # pretend you got some data from a sensor\n\n\nscistorage.write(info, data, "data.pkl.gz") # *.gz uses gzip compression\n\nbag = scistorage.read("data.pkl.gz")\nprint(bag["info"])\nprint(bag["data"])\n```\n\n## Network\n\n```python\nfrom slurm import network\n\nprint(network.get_ip()) # -> ip_address\nprint(network.host()) # -> (hostname, ip_address)\n```\n\n## Sleep Rate\n\nWill sleep for a prescribed amount of time inside of a loop\nirregardless of how long the loop takes\n\n```python\nfrom slurm import Rate\n\nrate = Rate(10)  # let loop run at 10 Hz\n\nwhile True:\n    # do some processing\n    rate.sleep()\n```\n\n## Files\n\n```python\nfrom slurm.files import rmdir, mkdir, run, rm, find\n\nmkdir("some/path")\nrmdir("some/path")\nrm("/path/file.txt")\nrm(["path/a.txt", "path/2/b.txt", "path/3/c.txt"])\n\nfind("/path/to/somewhere", "file_to_find") # -> list\nfind("/path/to/somewhere", "*.html") # -> list\n\nrun("ls -alh") # -> output\n```\n\n# MIT License\n\n**Copyright (c) 2014 Kevin J. Walchko**\n\nPermission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.\n',
    'author': 'walchko',
    'author_email': 'walchko@users.noreply.github.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://pypi.org/project/slurm/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8',
}


setup(**setup_kwargs)
